# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo
from .commission_details_param import CommissionDetailsParam

__all__ = ["VirtualAccountUpdateParams", "Variant0", "Variant1"]


class Variant0(TypedDict, total=False):
    action: Required[Literal["update"]]
    """The type of action to take.

    An action of type "update" will update the virtual account with the specified
    properties.
    """

    commission_details: Annotated[CommissionDetailsParam, PropertyInfo(alias="commissionDetails")]

    name: str
    """The name to update the virtual account with."""


class Variant1(TypedDict, total=False):
    action: Required[Literal["close"]]
    """The type of action to take.

    An action of type "close" will close the virtual account.
    """


VirtualAccountUpdateParams: TypeAlias = Union[Variant0, Variant1]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo
from .card.spending_constraint_param import SpendingConstraintParam

__all__ = ["CardGroupUpdateParams"]


class CardGroupUpdateParams(TypedDict, total=False):
    name: str

    spending_constraint: Annotated[Optional[SpendingConstraintParam], PropertyInfo(alias="spendingConstraint")]
    """
    Explicitly set this value to null to remove all card group level spending
    constraints.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["StatementListResponse", "Statement", "StatementLink", "Page"]


class StatementLink(BaseModel):
    href: str

    action: Optional[Literal["GET"]] = None

    rel: Optional[str] = None

    types: Optional[List[Literal["application/pdf"]]] = None


class Statement(BaseModel):
    account_id: str = FieldInfo(alias="accountId")
    """Long-term persistent identity of the account, though not an account number."""

    description: str
    """A description of the statement."""

    links: List[StatementLink]
    """
    Will only contain one element, which is a link to the
    /accounts/{accountId}/statements/{statementId} endpoint.
    """

    statement_date: str = FieldInfo(alias="statementDate")
    """The date of the statement."""

    statement_id: str = FieldInfo(alias="statementId")
    """Long-term persistent identity of the statement."""

    status: Literal["pending", "available"]


class Page(BaseModel):
    next_offset: Optional[str] = FieldInfo(alias="nextOffset", default=None)


class StatementListResponse(BaseModel):
    statements: List[Statement]

    page: Optional[Page] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .statements import (
    StatementsResource,
    AsyncStatementsResource,
    StatementsResourceWithRawResponse,
    AsyncStatementsResourceWithRawResponse,
    StatementsResourceWithStreamingResponse,
    AsyncStatementsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....types.fdx import (
    account_list_params,
    account_list_transactions_params,
    account_retrieve_payment_networks_params,
)
from ...._base_client import make_request_options
from ....types.fdx.account_list_response import AccountListResponse
from ....types.fdx.account_retrieve_response import AccountRetrieveResponse
from ....types.fdx.account_retrieve_contact_response import AccountRetrieveContactResponse
from ....types.fdx.account_list_transactions_response import AccountListTransactionsResponse
from ....types.fdx.account_retrieve_payment_networks_response import AccountRetrievePaymentNetworksResponse

__all__ = ["AccountsResource", "AsyncAccountsResource"]


class AccountsResource(SyncAPIResource):
    @cached_property
    def statements(self) -> StatementsResource:
        return StatementsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AccountsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveResponse:
        """
        Get account balances and liabilities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            AccountRetrieveResponse,
            self._get(
                f"/fdx/accounts/{account_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, AccountRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        limit: str | Omit = omit,
        offset: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountListResponse:
        """
        List all accounts

        Args:
          limit: The number of items to return (default 100, max 1000)

          offset: The number of items to skip before starting to collect the result set

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/fdx/accounts",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            cast_to=AccountListResponse,
        )

    def list_transactions(
        self,
        account_id: str,
        *,
        end_time: str | Omit = omit,
        limit: str | Omit = omit,
        offset: str | Omit = omit,
        start_time: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountListTransactionsResponse:
        """
        Search for transactions

        Args:
          end_time: End time for use in retrieval of transactions (ISO 8601).

          limit: The number of items to return (default 100, max 1000)

          offset: The ID of the last item in the previous page of results

          start_time: Start time for use in retrieval of transactions (ISO 8601).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/fdx/accounts/{account_id}/transactions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_time": end_time,
                        "limit": limit,
                        "offset": offset,
                        "start_time": start_time,
                    },
                    account_list_transactions_params.AccountListTransactionsParams,
                ),
            ),
            cast_to=AccountListTransactionsResponse,
        )

    def retrieve_contact(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveContactResponse:
        """
        Get an account's contact information

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/fdx/accounts/{account_id}/contact",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountRetrieveContactResponse,
        )

    def retrieve_payment_networks(
        self,
        account_id: str,
        *,
        limit: str | Omit = omit,
        offset: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrievePaymentNetworksResponse:
        """
        Get payment networks supported by the account

        Args:
          limit: The number of items to return (default 100, max 1000)

          offset: The ID of the last item in the previous page of results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/fdx/accounts/{account_id}/payment-networks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    account_retrieve_payment_networks_params.AccountRetrievePaymentNetworksParams,
                ),
            ),
            cast_to=AccountRetrievePaymentNetworksResponse,
        )


class AsyncAccountsResource(AsyncAPIResource):
    @cached_property
    def statements(self) -> AsyncStatementsResource:
        return AsyncStatementsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncAccountsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveResponse:
        """
        Get account balances and liabilities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            AccountRetrieveResponse,
            await self._get(
                f"/fdx/accounts/{account_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, AccountRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def list(
        self,
        *,
        limit: str | Omit = omit,
        offset: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountListResponse:
        """
        List all accounts

        Args:
          limit: The number of items to return (default 100, max 1000)

          offset: The number of items to skip before starting to collect the result set

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/fdx/accounts",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            cast_to=AccountListResponse,
        )

    async def list_transactions(
        self,
        account_id: str,
        *,
        end_time: str | Omit = omit,
        limit: str | Omit = omit,
        offset: str | Omit = omit,
        start_time: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountListTransactionsResponse:
        """
        Search for transactions

        Args:
          end_time: End time for use in retrieval of transactions (ISO 8601).

          limit: The number of items to return (default 100, max 1000)

          offset: The ID of the last item in the previous page of results

          start_time: Start time for use in retrieval of transactions (ISO 8601).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/fdx/accounts/{account_id}/transactions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_time": end_time,
                        "limit": limit,
                        "offset": offset,
                        "start_time": start_time,
                    },
                    account_list_transactions_params.AccountListTransactionsParams,
                ),
            ),
            cast_to=AccountListTransactionsResponse,
        )

    async def retrieve_contact(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveContactResponse:
        """
        Get an account's contact information

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/fdx/accounts/{account_id}/contact",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountRetrieveContactResponse,
        )

    async def retrieve_payment_networks(
        self,
        account_id: str,
        *,
        limit: str | Omit = omit,
        offset: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrievePaymentNetworksResponse:
        """
        Get payment networks supported by the account

        Args:
          limit: The number of items to return (default 100, max 1000)

          offset: The ID of the last item in the previous page of results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/fdx/accounts/{account_id}/payment-networks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    account_retrieve_payment_networks_params.AccountRetrievePaymentNetworksParams,
                ),
            ),
            cast_to=AccountRetrievePaymentNetworksResponse,
        )


class AccountsResourceWithRawResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = to_raw_response_wrapper(
            accounts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            accounts.list,
        )
        self.list_transactions = to_raw_response_wrapper(
            accounts.list_transactions,
        )
        self.retrieve_contact = to_raw_response_wrapper(
            accounts.retrieve_contact,
        )
        self.retrieve_payment_networks = to_raw_response_wrapper(
            accounts.retrieve_payment_networks,
        )

    @cached_property
    def statements(self) -> StatementsResourceWithRawResponse:
        return StatementsResourceWithRawResponse(self._accounts.statements)


class AsyncAccountsResourceWithRawResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = async_to_raw_response_wrapper(
            accounts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            accounts.list,
        )
        self.list_transactions = async_to_raw_response_wrapper(
            accounts.list_transactions,
        )
        self.retrieve_contact = async_to_raw_response_wrapper(
            accounts.retrieve_contact,
        )
        self.retrieve_payment_networks = async_to_raw_response_wrapper(
            accounts.retrieve_payment_networks,
        )

    @cached_property
    def statements(self) -> AsyncStatementsResourceWithRawResponse:
        return AsyncStatementsResourceWithRawResponse(self._accounts.statements)


class AccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = to_streamed_response_wrapper(
            accounts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            accounts.list,
        )
        self.list_transactions = to_streamed_response_wrapper(
            accounts.list_transactions,
        )
        self.retrieve_contact = to_streamed_response_wrapper(
            accounts.retrieve_contact,
        )
        self.retrieve_payment_networks = to_streamed_response_wrapper(
            accounts.retrieve_payment_networks,
        )

    @cached_property
    def statements(self) -> StatementsResourceWithStreamingResponse:
        return StatementsResourceWithStreamingResponse(self._accounts.statements)


class AsyncAccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = async_to_streamed_response_wrapper(
            accounts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            accounts.list,
        )
        self.list_transactions = async_to_streamed_response_wrapper(
            accounts.list_transactions,
        )
        self.retrieve_contact = async_to_streamed_response_wrapper(
            accounts.retrieve_contact,
        )
        self.retrieve_payment_networks = async_to_streamed_response_wrapper(
            accounts.retrieve_payment_networks,
        )

    @cached_property
    def statements(self) -> AsyncStatementsResourceWithStreamingResponse:
        return AsyncStatementsResourceWithStreamingResponse(self._accounts.statements)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo
from .restriction import Restriction
from ..money_param import MoneyParam

__all__ = [
    "SpendingConstraintParam",
    "CountryRule",
    "MerchantCategoryCodeRule",
    "MerchantCategoryRule",
    "MerchantRule",
    "SpendingRule",
    "SpendingRuleTransactionSizeLimit",
    "SpendingRuleUtilizationLimit",
    "SpendingRuleUtilizationLimitV2",
]


class CountryRule(TypedDict, total=False):
    countries: Required[SequenceNotStr[str]]
    """A 2 digit country code"""

    restriction: Required[Restriction]


class MerchantCategoryCodeRule(TypedDict, total=False):
    merchant_category_codes: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="merchantCategoryCodes")]]

    restriction: Required[Restriction]


class MerchantCategoryRule(TypedDict, total=False):
    merchant_categories: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="merchantCategories")]]

    restriction: Required[Restriction]


class MerchantRule(TypedDict, total=False):
    merchants: Required[SequenceNotStr[str]]

    restriction: Required[Restriction]


class SpendingRuleTransactionSizeLimit(TypedDict, total=False):
    maximum: MoneyParam
    """Represents a monetary value"""

    minimum: MoneyParam
    """Represents a monetary value"""


class SpendingRuleUtilizationLimit(TypedDict, total=False):
    limit_amount: Required[Annotated[MoneyParam, PropertyInfo(alias="limitAmount")]]
    """Represents a monetary value"""

    preset: Required[Literal["daily", "weekly", "monthly", "yearly", "collective"]]

    start_date: Annotated[str, PropertyInfo(alias="startDate")]
    """Format ISO-8601.

    A day that equals today or the past. This is optional.If the `preset` is
    "daily", this value is ignored. If the `preset` is "weekly", "monthly" or
    "yearly", then the this value is used to compute when the limit should start
    limit.
    """

    timezone: str
    """IANA timezone string.

    Limits always reset at midnight of the timezone specified. If no timezone is
    specified, then UTC time is used.
    """


class SpendingRuleUtilizationLimitV2(TypedDict, total=False):
    limit_amount: Required[Annotated[MoneyParam, PropertyInfo(alias="limitAmount")]]
    """Represents a monetary value"""

    preset: Required[Literal["daily", "weekly", "monthly", "yearly", "collective"]]

    start_date: Annotated[str, PropertyInfo(alias="startDate")]
    """Format ISO-8601.

    A day that equals today or the past. This is optional.If the `preset` is
    "daily", this value is ignored. If the `preset` is "weekly", "monthly" or
    "yearly", then the this value is used to compute when the limit should start
    limit.
    """

    timezone: str
    """IANA timezone string.

    Limits always reset at midnight of the timezone specified. If no timezone is
    specified, then UTC time is used.
    """


class SpendingRule(TypedDict, total=False):
    transaction_size_limit: Annotated[SpendingRuleTransactionSizeLimit, PropertyInfo(alias="transactionSizeLimit")]

    utilization_limit: Annotated[SpendingRuleUtilizationLimit, PropertyInfo(alias="utilizationLimit")]

    utilization_limit_v2: Annotated[Iterable[SpendingRuleUtilizationLimitV2], PropertyInfo(alias="utilizationLimitV2")]


class SpendingConstraintParam(TypedDict, total=False):
    """A constraint that can be applied to a CardGroupSpendingRule"""

    country_rule: Annotated[Optional[CountryRule], PropertyInfo(alias="countryRule")]

    merchant_category_code_rule: Annotated[
        Optional[MerchantCategoryCodeRule], PropertyInfo(alias="merchantCategoryCodeRule")
    ]

    merchant_category_rule: Annotated[Optional[MerchantCategoryRule], PropertyInfo(alias="merchantCategoryRule")]

    merchant_rule: Annotated[Optional[MerchantRule], PropertyInfo(alias="merchantRule")]

    spending_rule: Annotated[Optional[SpendingRule], PropertyInfo(alias="spendingRule")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types.fdx import (
    AccountListResponse,
    AccountRetrieveResponse,
    AccountRetrieveContactResponse,
    AccountListTransactionsResponse,
    AccountRetrievePaymentNetworksResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAccounts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: SlashSDK) -> None:
        account = client.fdx.accounts.retrieve(
            "accountId",
        )
        assert_matches_type(AccountRetrieveResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: SlashSDK) -> None:
        response = client.fdx.accounts.with_raw_response.retrieve(
            "accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountRetrieveResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: SlashSDK) -> None:
        with client.fdx.accounts.with_streaming_response.retrieve(
            "accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountRetrieveResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.fdx.accounts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: SlashSDK) -> None:
        account = client.fdx.accounts.list()
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: SlashSDK) -> None:
        account = client.fdx.accounts.list(
            limit="limit",
            offset="offset",
        )
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: SlashSDK) -> None:
        response = client.fdx.accounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: SlashSDK) -> None:
        with client.fdx.accounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountListResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_transactions(self, client: SlashSDK) -> None:
        account = client.fdx.accounts.list_transactions(
            account_id="accountId",
        )
        assert_matches_type(AccountListTransactionsResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_transactions_with_all_params(self, client: SlashSDK) -> None:
        account = client.fdx.accounts.list_transactions(
            account_id="accountId",
            end_time="endTime",
            limit="limit",
            offset="offset",
            start_time="startTime",
        )
        assert_matches_type(AccountListTransactionsResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_transactions(self, client: SlashSDK) -> None:
        response = client.fdx.accounts.with_raw_response.list_transactions(
            account_id="accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountListTransactionsResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_transactions(self, client: SlashSDK) -> None:
        with client.fdx.accounts.with_streaming_response.list_transactions(
            account_id="accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountListTransactionsResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_transactions(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.fdx.accounts.with_raw_response.list_transactions(
                account_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_contact(self, client: SlashSDK) -> None:
        account = client.fdx.accounts.retrieve_contact(
            "accountId",
        )
        assert_matches_type(AccountRetrieveContactResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve_contact(self, client: SlashSDK) -> None:
        response = client.fdx.accounts.with_raw_response.retrieve_contact(
            "accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountRetrieveContactResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_contact(self, client: SlashSDK) -> None:
        with client.fdx.accounts.with_streaming_response.retrieve_contact(
            "accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountRetrieveContactResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve_contact(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.fdx.accounts.with_raw_response.retrieve_contact(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_payment_networks(self, client: SlashSDK) -> None:
        account = client.fdx.accounts.retrieve_payment_networks(
            account_id="accountId",
        )
        assert_matches_type(AccountRetrievePaymentNetworksResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_payment_networks_with_all_params(self, client: SlashSDK) -> None:
        account = client.fdx.accounts.retrieve_payment_networks(
            account_id="accountId",
            limit="limit",
            offset="offset",
        )
        assert_matches_type(AccountRetrievePaymentNetworksResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve_payment_networks(self, client: SlashSDK) -> None:
        response = client.fdx.accounts.with_raw_response.retrieve_payment_networks(
            account_id="accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountRetrievePaymentNetworksResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_payment_networks(self, client: SlashSDK) -> None:
        with client.fdx.accounts.with_streaming_response.retrieve_payment_networks(
            account_id="accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountRetrievePaymentNetworksResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve_payment_networks(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.fdx.accounts.with_raw_response.retrieve_payment_networks(
                account_id="",
            )


class TestAsyncAccounts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.fdx.accounts.retrieve(
            "accountId",
        )
        assert_matches_type(AccountRetrieveResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.fdx.accounts.with_raw_response.retrieve(
            "accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountRetrieveResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.fdx.accounts.with_streaming_response.retrieve(
            "accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountRetrieveResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.fdx.accounts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.fdx.accounts.list()
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.fdx.accounts.list(
            limit="limit",
            offset="offset",
        )
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.fdx.accounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.fdx.accounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountListResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_transactions(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.fdx.accounts.list_transactions(
            account_id="accountId",
        )
        assert_matches_type(AccountListTransactionsResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_transactions_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.fdx.accounts.list_transactions(
            account_id="accountId",
            end_time="endTime",
            limit="limit",
            offset="offset",
            start_time="startTime",
        )
        assert_matches_type(AccountListTransactionsResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_transactions(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.fdx.accounts.with_raw_response.list_transactions(
            account_id="accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountListTransactionsResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_transactions(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.fdx.accounts.with_streaming_response.list_transactions(
            account_id="accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountListTransactionsResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_transactions(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.fdx.accounts.with_raw_response.list_transactions(
                account_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_contact(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.fdx.accounts.retrieve_contact(
            "accountId",
        )
        assert_matches_type(AccountRetrieveContactResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_contact(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.fdx.accounts.with_raw_response.retrieve_contact(
            "accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountRetrieveContactResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_contact(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.fdx.accounts.with_streaming_response.retrieve_contact(
            "accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountRetrieveContactResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve_contact(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.fdx.accounts.with_raw_response.retrieve_contact(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_payment_networks(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.fdx.accounts.retrieve_payment_networks(
            account_id="accountId",
        )
        assert_matches_type(AccountRetrievePaymentNetworksResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_payment_networks_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.fdx.accounts.retrieve_payment_networks(
            account_id="accountId",
            limit="limit",
            offset="offset",
        )
        assert_matches_type(AccountRetrievePaymentNetworksResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_payment_networks(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.fdx.accounts.with_raw_response.retrieve_payment_networks(
            account_id="accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountRetrievePaymentNetworksResponse, account, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_payment_networks(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.fdx.accounts.with_streaming_response.retrieve_payment_networks(
            account_id="accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountRetrievePaymentNetworksResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve_payment_networks(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.fdx.accounts.with_raw_response.retrieve_payment_networks(
                account_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import account_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.account import Account
from ..types.account_list_response import AccountListResponse
from ..types.account_retrieve_balance_response import AccountRetrieveBalanceResponse

__all__ = ["AccountResource", "AsyncAccountResource"]


class AccountResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AccountResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AccountResourceWithStreamingResponse(self)

    def retrieve(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Account:
        """
        Get a single account by Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/account/{account_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Account,
        )

    def list(
        self,
        *,
        filter_legal_entity_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountListResponse:
        """
        List all accounts you have access to

        Args:
          filter_legal_entity_id: Pass in a legal entity ID to filter for accounts under a specific legal entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/account",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"filter_legal_entity_id": filter_legal_entity_id}, account_list_params.AccountListParams
                ),
            ),
            cast_to=AccountListResponse,
        )

    def retrieve_balance(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveBalanceResponse:
        """Retrieve a list of balances for an account.

        The balances are returned in an
        array, with each item representing a balance for that account.

        For charge_card accounts, there will be two balances returned: "cash" and
        "credit". For debit accounts, there will only be one balance returned: "debit".

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/account/{account_id}/balance",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountRetrieveBalanceResponse,
        )


class AsyncAccountResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncAccountResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Account:
        """
        Get a single account by Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/account/{account_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Account,
        )

    async def list(
        self,
        *,
        filter_legal_entity_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountListResponse:
        """
        List all accounts you have access to

        Args:
          filter_legal_entity_id: Pass in a legal entity ID to filter for accounts under a specific legal entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/account",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter_legal_entity_id": filter_legal_entity_id}, account_list_params.AccountListParams
                ),
            ),
            cast_to=AccountListResponse,
        )

    async def retrieve_balance(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveBalanceResponse:
        """Retrieve a list of balances for an account.

        The balances are returned in an
        array, with each item representing a balance for that account.

        For charge_card accounts, there will be two balances returned: "cash" and
        "credit". For debit accounts, there will only be one balance returned: "debit".

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/account/{account_id}/balance",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountRetrieveBalanceResponse,
        )


class AccountResourceWithRawResponse:
    def __init__(self, account: AccountResource) -> None:
        self._account = account

        self.retrieve = to_raw_response_wrapper(
            account.retrieve,
        )
        self.list = to_raw_response_wrapper(
            account.list,
        )
        self.retrieve_balance = to_raw_response_wrapper(
            account.retrieve_balance,
        )


class AsyncAccountResourceWithRawResponse:
    def __init__(self, account: AsyncAccountResource) -> None:
        self._account = account

        self.retrieve = async_to_raw_response_wrapper(
            account.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            account.list,
        )
        self.retrieve_balance = async_to_raw_response_wrapper(
            account.retrieve_balance,
        )


class AccountResourceWithStreamingResponse:
    def __init__(self, account: AccountResource) -> None:
        self._account = account

        self.retrieve = to_streamed_response_wrapper(
            account.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            account.list,
        )
        self.retrieve_balance = to_streamed_response_wrapper(
            account.retrieve_balance,
        )


class AsyncAccountResourceWithStreamingResponse:
    def __init__(self, account: AsyncAccountResource) -> None:
        self._account = account

        self.retrieve = async_to_streamed_response_wrapper(
            account.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            account.list,
        )
        self.retrieve_balance = async_to_streamed_response_wrapper(
            account.retrieve_balance,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .developer_application_type import DeveloperApplicationType
from .developer_application_data_param import DeveloperApplicationDataParam

__all__ = ["DeveloperAccountCreateApplicationParams"]


class DeveloperAccountCreateApplicationParams(TypedDict, total=False):
    data: Required[DeveloperApplicationDataParam]

    name: Required[str]

    type: Required[DeveloperApplicationType]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .card.spending_constraint_param import SpendingConstraintParam

__all__ = ["CardGroupCreateParams"]


class CardGroupCreateParams(TypedDict, total=False):
    name: Required[str]

    spending_constraint: Annotated[SpendingConstraintParam, PropertyInfo(alias="spendingConstraint")]
    """A constraint that can be applied to a CardGroupSpendingRule"""

    virtual_account_id: Annotated[str, PropertyInfo(alias="virtualAccountId")]
    """The ID of the virtual account to associate with the card group.

    If not provided, the primary virtual account will be used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["VirtualAccountListParams"]


class VirtualAccountListParams(TypedDict, total=False):
    cursor: str
    """A cursor string to fetch the next page of results"""

    filter_account_id: Annotated[str, PropertyInfo(alias="filter:accountId")]
    """Pass in an account ID to filter virtual accounts by account ID.

    This will return all virtual accounts that match the account ID passed in.
    """

    filter_include_closed_accounts: Annotated[Literal, PropertyInfo(alias="filter:includeClosedAccounts")]
    """Include virtual accounts that have been closed in the query results.

    By default, they will not be returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .slash_handle import SlashHandle
from .pagination_response import PaginationResponse

__all__ = ["SlashHandleListResponse"]


class SlashHandleListResponse(BaseModel):
    items: List[SlashHandle]

    metadata: PaginationResponse
    """Response sent when requesting a list of data"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from ...types import CardStatus, card_list_params, card_create_params, card_update_params, card_retrieve_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.card.card import Card
from ...types.card_status import CardStatus
from .spending_constraint import (
    SpendingConstraintResource,
    AsyncSpendingConstraintResource,
    SpendingConstraintResourceWithRawResponse,
    AsyncSpendingConstraintResourceWithRawResponse,
    SpendingConstraintResourceWithStreamingResponse,
    AsyncSpendingConstraintResourceWithStreamingResponse,
)
from ...types.card_list_response import CardListResponse
from ...types.card_group_utilization import CardGroupUtilization
from ...types.card.spending_constraint_param import SpendingConstraintParam

__all__ = ["CardResource", "AsyncCardResource"]


class CardResource(SyncAPIResource):
    @cached_property
    def spending_constraint(self) -> SpendingConstraintResource:
        return SpendingConstraintResource(self._client)

    @cached_property
    def with_raw_response(self) -> CardResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CardResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CardResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return CardResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        type: Literal["virtual"],
        account_id: str | Omit = omit,
        card_group_id: str | Omit = omit,
        card_product_id: str | Omit = omit,
        is_single_use: bool | Omit = omit,
        spending_constraint: SpendingConstraintParam | Omit = omit,
        user_data: Dict[str, object] | Omit = omit,
        virtual_account_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Card:
        """Create a card

        Args:
          type: Specify the type of card you'd like to create.

        At the moment, only virtual cards
              are supported.

          account_id: The ID of the account to create the card under. You can get this by calling
              `GET /account`. This field is required unless you are authenticating via API
              key, in which case it will default to your first commercial account. We
              recommend supplying this even if you are authenticating via API key.

          card_product_id: The ID of the card product to use when creating this card, if not specified a
              random card product will be chosen.

          is_single_use: Defaults to false. When set to true, the card will be automatically closed after
              a single authorization attempt. Note that the card will be closed even if the
              authorization declines or drops

          spending_constraint: A constraint that can be applied to a CardGroupSpendingRule

          user_data: Arbitrary information that can be attached to the card. This should be a JSON
              object and cannot exceed 4kb.

          virtual_account_id: The ID of the virtual account to create the card under. Virtual accounts can be
              retrieved by calling `GET /virtual-account`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/card",
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "account_id": account_id,
                    "card_group_id": card_group_id,
                    "card_product_id": card_product_id,
                    "is_single_use": is_single_use,
                    "spending_constraint": spending_constraint,
                    "user_data": user_data,
                    "virtual_account_id": virtual_account_id,
                },
                card_create_params.CardCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Card,
        )

    def retrieve(
        self,
        card_id: str,
        *,
        include_cvv: Literal["true", "false"] | Omit = omit,
        include_pan: Literal["true", "false"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Card:
        """
        Fetch details for a single card by card ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return self._get(
            f"/card/{card_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_cvv": include_cvv,
                        "include_pan": include_pan,
                    },
                    card_retrieve_params.CardRetrieveParams,
                ),
            ),
            cast_to=Card,
        )

    def update(
        self,
        card_id: str,
        *,
        card_group_id: Optional[str] | Omit = omit,
        name: str | Omit = omit,
        spending_constraint: Optional[SpendingConstraintParam] | Omit = omit,
        status: CardStatus | Omit = omit,
        user_data: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Card:
        """
        Update a card

        Args:
          card_group_id: Explicitly set this value to null to remove the card from a group. Omitting this
              field entirely will not affect the group the card belongs to.

          spending_constraint: Explicitly set this value to null to remove all card level spending constraints.

          user_data: Arbitrary information that can be attached to the card. This should be a JSON
              object and cannot exceed 4kb.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return self._patch(
            f"/card/{card_id}",
            body=maybe_transform(
                {
                    "card_group_id": card_group_id,
                    "name": name,
                    "spending_constraint": spending_constraint,
                    "status": status,
                    "user_data": user_data,
                },
                card_update_params.CardUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Card,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        filter_account_id: str | Omit = omit,
        filter_card_group_id: str | Omit = omit,
        filter_card_group_name: str | Omit = omit,
        filter_legal_entity_id: str | Omit = omit,
        filter_status: Literal["active", "paused", "closed", "inactive"] | Omit = omit,
        filter_virtual_account_id: str | Omit = omit,
        sort: Literal["createdAt", "name"] | Omit = omit,
        sort_direction: Literal["ASC", "DESC"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardListResponse:
        """
        List all cards you have access to.

        Args:
          cursor: A cursor string to fetch the next page of results

          filter_account_id: Pass in an account ID to filter for cards under a specific account.

          filter_card_group_id: Pass in a card group ID, This will return all cards that belong to the card
              group ID passed in. Cannot be combined with filter:cardGroupName.

          filter_card_group_name: Pass in a card group name, This will return all cards that belong to the card
              group name passed in. Cannot be combined with filter:cardGroupId.

          filter_legal_entity_id: Pass in a legal entity ID to filter for cards in accounts under a specific legal
              entity.

          filter_status: Returns all cards matching the status passed in.

          filter_virtual_account_id: Pass in a virtual account ID to filter for cards under a specific virtual
              account.

          sort: Sorts card by creation date or name.

          sort_direction: The direction to apply the sort filter by. Default ASC.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/card",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "filter_account_id": filter_account_id,
                        "filter_card_group_id": filter_card_group_id,
                        "filter_card_group_name": filter_card_group_name,
                        "filter_legal_entity_id": filter_legal_entity_id,
                        "filter_status": filter_status,
                        "filter_virtual_account_id": filter_virtual_account_id,
                        "sort": sort,
                        "sort_direction": sort_direction,
                    },
                    card_list_params.CardListParams,
                ),
            ),
            cast_to=CardListResponse,
        )

    def get_utilization(
        self,
        card_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroupUtilization:
        """
        Get a card's current utilization

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return self._get(
            f"/card/{card_id}/utilization",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroupUtilization,
        )


class AsyncCardResource(AsyncAPIResource):
    @cached_property
    def spending_constraint(self) -> AsyncSpendingConstraintResource:
        return AsyncSpendingConstraintResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCardResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCardResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCardResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncCardResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        type: Literal["virtual"],
        account_id: str | Omit = omit,
        card_group_id: str | Omit = omit,
        card_product_id: str | Omit = omit,
        is_single_use: bool | Omit = omit,
        spending_constraint: SpendingConstraintParam | Omit = omit,
        user_data: Dict[str, object] | Omit = omit,
        virtual_account_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Card:
        """Create a card

        Args:
          type: Specify the type of card you'd like to create.

        At the moment, only virtual cards
              are supported.

          account_id: The ID of the account to create the card under. You can get this by calling
              `GET /account`. This field is required unless you are authenticating via API
              key, in which case it will default to your first commercial account. We
              recommend supplying this even if you are authenticating via API key.

          card_product_id: The ID of the card product to use when creating this card, if not specified a
              random card product will be chosen.

          is_single_use: Defaults to false. When set to true, the card will be automatically closed after
              a single authorization attempt. Note that the card will be closed even if the
              authorization declines or drops

          spending_constraint: A constraint that can be applied to a CardGroupSpendingRule

          user_data: Arbitrary information that can be attached to the card. This should be a JSON
              object and cannot exceed 4kb.

          virtual_account_id: The ID of the virtual account to create the card under. Virtual accounts can be
              retrieved by calling `GET /virtual-account`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/card",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "account_id": account_id,
                    "card_group_id": card_group_id,
                    "card_product_id": card_product_id,
                    "is_single_use": is_single_use,
                    "spending_constraint": spending_constraint,
                    "user_data": user_data,
                    "virtual_account_id": virtual_account_id,
                },
                card_create_params.CardCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Card,
        )

    async def retrieve(
        self,
        card_id: str,
        *,
        include_cvv: Literal["true", "false"] | Omit = omit,
        include_pan: Literal["true", "false"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Card:
        """
        Fetch details for a single card by card ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return await self._get(
            f"/card/{card_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_cvv": include_cvv,
                        "include_pan": include_pan,
                    },
                    card_retrieve_params.CardRetrieveParams,
                ),
            ),
            cast_to=Card,
        )

    async def update(
        self,
        card_id: str,
        *,
        card_group_id: Optional[str] | Omit = omit,
        name: str | Omit = omit,
        spending_constraint: Optional[SpendingConstraintParam] | Omit = omit,
        status: CardStatus | Omit = omit,
        user_data: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Card:
        """
        Update a card

        Args:
          card_group_id: Explicitly set this value to null to remove the card from a group. Omitting this
              field entirely will not affect the group the card belongs to.

          spending_constraint: Explicitly set this value to null to remove all card level spending constraints.

          user_data: Arbitrary information that can be attached to the card. This should be a JSON
              object and cannot exceed 4kb.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return await self._patch(
            f"/card/{card_id}",
            body=await async_maybe_transform(
                {
                    "card_group_id": card_group_id,
                    "name": name,
                    "spending_constraint": spending_constraint,
                    "status": status,
                    "user_data": user_data,
                },
                card_update_params.CardUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Card,
        )

    async def list(
        self,
        *,
        cursor: str | Omit = omit,
        filter_account_id: str | Omit = omit,
        filter_card_group_id: str | Omit = omit,
        filter_card_group_name: str | Omit = omit,
        filter_legal_entity_id: str | Omit = omit,
        filter_status: Literal["active", "paused", "closed", "inactive"] | Omit = omit,
        filter_virtual_account_id: str | Omit = omit,
        sort: Literal["createdAt", "name"] | Omit = omit,
        sort_direction: Literal["ASC", "DESC"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardListResponse:
        """
        List all cards you have access to.

        Args:
          cursor: A cursor string to fetch the next page of results

          filter_account_id: Pass in an account ID to filter for cards under a specific account.

          filter_card_group_id: Pass in a card group ID, This will return all cards that belong to the card
              group ID passed in. Cannot be combined with filter:cardGroupName.

          filter_card_group_name: Pass in a card group name, This will return all cards that belong to the card
              group name passed in. Cannot be combined with filter:cardGroupId.

          filter_legal_entity_id: Pass in a legal entity ID to filter for cards in accounts under a specific legal
              entity.

          filter_status: Returns all cards matching the status passed in.

          filter_virtual_account_id: Pass in a virtual account ID to filter for cards under a specific virtual
              account.

          sort: Sorts card by creation date or name.

          sort_direction: The direction to apply the sort filter by. Default ASC.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/card",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "filter_account_id": filter_account_id,
                        "filter_card_group_id": filter_card_group_id,
                        "filter_card_group_name": filter_card_group_name,
                        "filter_legal_entity_id": filter_legal_entity_id,
                        "filter_status": filter_status,
                        "filter_virtual_account_id": filter_virtual_account_id,
                        "sort": sort,
                        "sort_direction": sort_direction,
                    },
                    card_list_params.CardListParams,
                ),
            ),
            cast_to=CardListResponse,
        )

    async def get_utilization(
        self,
        card_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardGroupUtilization:
        """
        Get a card's current utilization

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return await self._get(
            f"/card/{card_id}/utilization",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardGroupUtilization,
        )


class CardResourceWithRawResponse:
    def __init__(self, card: CardResource) -> None:
        self._card = card

        self.create = to_raw_response_wrapper(
            card.create,
        )
        self.retrieve = to_raw_response_wrapper(
            card.retrieve,
        )
        self.update = to_raw_response_wrapper(
            card.update,
        )
        self.list = to_raw_response_wrapper(
            card.list,
        )
        self.get_utilization = to_raw_response_wrapper(
            card.get_utilization,
        )

    @cached_property
    def spending_constraint(self) -> SpendingConstraintResourceWithRawResponse:
        return SpendingConstraintResourceWithRawResponse(self._card.spending_constraint)


class AsyncCardResourceWithRawResponse:
    def __init__(self, card: AsyncCardResource) -> None:
        self._card = card

        self.create = async_to_raw_response_wrapper(
            card.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            card.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            card.update,
        )
        self.list = async_to_raw_response_wrapper(
            card.list,
        )
        self.get_utilization = async_to_raw_response_wrapper(
            card.get_utilization,
        )

    @cached_property
    def spending_constraint(self) -> AsyncSpendingConstraintResourceWithRawResponse:
        return AsyncSpendingConstraintResourceWithRawResponse(self._card.spending_constraint)


class CardResourceWithStreamingResponse:
    def __init__(self, card: CardResource) -> None:
        self._card = card

        self.create = to_streamed_response_wrapper(
            card.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            card.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            card.update,
        )
        self.list = to_streamed_response_wrapper(
            card.list,
        )
        self.get_utilization = to_streamed_response_wrapper(
            card.get_utilization,
        )

    @cached_property
    def spending_constraint(self) -> SpendingConstraintResourceWithStreamingResponse:
        return SpendingConstraintResourceWithStreamingResponse(self._card.spending_constraint)


class AsyncCardResourceWithStreamingResponse:
    def __init__(self, card: AsyncCardResource) -> None:
        self._card = card

        self.create = async_to_streamed_response_wrapper(
            card.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            card.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            card.update,
        )
        self.list = async_to_streamed_response_wrapper(
            card.list,
        )
        self.get_utilization = async_to_streamed_response_wrapper(
            card.get_utilization,
        )

    @cached_property
    def spending_constraint(self) -> AsyncSpendingConstraintResourceWithStreamingResponse:
        return AsyncSpendingConstraintResourceWithStreamingResponse(self._card.spending_constraint)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["DeveloperApplicationDataParam"]


class DeveloperApplicationDataParam(TypedDict, total=False):
    allowed_origins: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="allowedOrigins")]]
    """
    The URLs or IP addresses that the application is allowed to make token exchange
    requests from. If the application makes a token exchange request from a URL that
    is not in this list, the request will be blocked.
    """

    description: Required[str]
    """
    A short description of the application that will be shown to users when they
    authorize the application.
    """

    scopes: Required[
        List[
            Literal[
                "legal_entity.home.view",
                "legal_entity.view",
                "legal_entity.users.view",
                "legal_entity.users.modify",
                "legal_entity.users.card_groups.view",
                "legal_entity.users.card_groups.modify",
                "legal_entity.subscriptions.view",
                "legal_entity.subscriptions.modify",
                "legal_entity.accounting.view",
                "legal_entity.accounting.modify",
                "legal_entity.contacts.view",
                "legal_entity.contacts.modify",
                "legal_entity.third_party_credentials.view",
                "legal_entity.third_party_credentials.modify",
                "legal_entity.authorized_third_parties.view",
                "legal_entity.authorized_third_parties.modify",
                "legal_entity.api_key.view",
                "legal_entity.api_key.modify",
                "legal_entity.applications.view",
                "legal_entity.applications.modify",
                "legal_entity.permission_roles.view",
                "legal_entity.automation_rules.view",
                "legal_entity.automation_rules.modify",
                "legal_entity.developer_account.view",
                "legal_entity.developer_account.modify",
                "legal_entity.cashback.view",
                "legal_entity.cashback.modify",
                "legal_entity.webhooks.view",
                "legal_entity.webhooks.modify",
                "legal_entity.ticket_reconciliation.view",
                "legal_entity.analytics.view",
                "legal_entity.transactions.crypto_deposit.accelerate",
                "slash_accounts.view",
                "slash_accounts.home.view",
                "slash_accounts.cards.physical.view",
                "slash_accounts.cards.physical.modify",
                "slash_accounts.cards.virtual.view",
                "slash_accounts.cards.virtual.modify",
                "slash_accounts.incoming_transactions.view",
                "slash_accounts.transactions.view",
                "slash_accounts.transactions.ach_pull.initiate",
                "slash_accounts.transactions.ach_push.initiate",
                "slash_accounts.transactions.wire.initiate",
                "slash_accounts.transactions.between_accounts.initiate",
                "slash_accounts.transactions.rtp.initiate",
                "slash_accounts.balance.view",
                "slash_accounts.subaccounts.view",
                "slash_accounts.subaccounts.modify",
                "slash_accounts.merchant_allowlist.view",
                "slash_accounts.merchant_allowlist.modify",
                "slash_accounts.integrations.view",
                "slash_accounts.integrations.modify",
            ]
        ]
    ]
    """The scopes that the application will request access to."""

    type: Required[Literal["service"]]

    logo_url: Annotated[str, PropertyInfo(alias="logoUrl")]
    """
    The URL of the application's logo that will be shown to users when they
    authorize the application.
    """

    privacy_policy_url: Annotated[str, PropertyInfo(alias="privacyPolicyUrl")]
    """
    The URL of the application's privacy policy that will be shown to users when
    they authorize the application.
    """

    redirect_uris: Annotated[SequenceNotStr[str], PropertyInfo(alias="redirectUris")]
    """
    The URLs of the application's redirect URIs that will be used when the
    application is authorized. The redirect URIs must match the ones registered
    here, or the authorization will fail.
    """

    terms_of_service_url: Annotated[str, PropertyInfo(alias="termsOfServiceUrl")]
    """
    The URL of the application's terms of service that will be shown to users when
    they authorize the application.
    """

    test_user_emails: Annotated[SequenceNotStr[str], PropertyInfo(alias="testUserEmails")]
    """
    A list of users that are allowed to test the application before it is approved
    for production. Max 10 users.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["WellKnownRetrieveOpenidConfigurationResponse"]


class WellKnownRetrieveOpenidConfigurationResponse(BaseModel):
    authorization_endpoint: str

    response_types_supported: List[Literal["code", "token", "id_token token"]]

    scopes_supported: List[
        Literal["openid", "offline_access", "profile", "email", "address", "phone", "user.identity.view"]
    ]

    token_endpoint: str

    userinfo_endpoint: str

    token_endpoint_auth_methods_supported: Optional[
        List[Literal["client_secret_post", "client_secret_basic", "client_secret_jwt", "private_key_jwt"]]
    ] = None
    """If omitted, the default is client_secret_basic"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import webhook_list_params, webhook_create_params, webhook_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.webhook import Webhook
from ..types.webhook_list_response import WebhookListResponse

__all__ = ["WebhookResource", "AsyncWebhookResource"]


class WebhookResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebhookResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return WebhookResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebhookResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return WebhookResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        url: str,
        legal_entity_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Webhook:
        """
        Create a new webhook endpoint

        Args:
          url: The URL that will receive the webhook payload

          legal_entity_id: The ID of the LegalEntity to create the webhook for. You can get this by calling
              `GET /legal-entity`. This field is required unless you are authenticating via
              API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/webhook",
            body=maybe_transform(
                {
                    "name": name,
                    "url": url,
                    "legal_entity_id": legal_entity_id,
                },
                webhook_create_params.WebhookCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Webhook,
        )

    def update(
        self,
        webhook_id: str,
        *,
        status: Literal["active", "paused", "archived"],
        reason: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Webhook:
        """
        Update a webhook endpoint status.

        **Status transitions:**

        - `active`: Re-enables the endpoint. Use this to:
          - Unpause a paused endpoint
          - Re-enable an endpoint that is backing-off or disabled due to delivery
            failures
          - When re-enabled, any queued notifications will be sent
        - `paused`: Pauses the endpoint. While paused:
          - New notifications are queued (not lost)
          - No delivery attempts are made
          - Use `active` to resume and process queued notifications
        - `archived`: Permanently archives the endpoint (soft delete)

        **Note:** The `backing-off` and `disabled` statuses are system-managed and
        cannot be set directly. These occur automatically when delivery failures are
        detected. Use `active` to re-enable.

        Args:
          status:
              The desired status for the endpoint:

              - `active`: Enable/re-enable the endpoint
              - `paused`: Pause the endpoint (notifications are queued)
              - `archived`: Archive the endpoint (soft delete)

          reason: Optional reason for the status change (for audit purposes)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._patch(
            f"/webhook/{webhook_id}",
            body=maybe_transform(
                {
                    "status": status,
                    "reason": reason,
                },
                webhook_update_params.WebhookUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Webhook,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        filter_legal_entity_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookListResponse:
        """
        Get all webhooks

        Args:
          cursor: A cursor string to fetch the next page of results

          filter_legal_entity_id: Pass in a legal entity ID to filter for webhooks under a specific legal entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/webhook",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "filter_legal_entity_id": filter_legal_entity_id,
                    },
                    webhook_list_params.WebhookListParams,
                ),
            ),
            cast_to=WebhookListResponse,
        )


class AsyncWebhookResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebhookResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWebhookResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebhookResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncWebhookResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        url: str,
        legal_entity_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Webhook:
        """
        Create a new webhook endpoint

        Args:
          url: The URL that will receive the webhook payload

          legal_entity_id: The ID of the LegalEntity to create the webhook for. You can get this by calling
              `GET /legal-entity`. This field is required unless you are authenticating via
              API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/webhook",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "url": url,
                    "legal_entity_id": legal_entity_id,
                },
                webhook_create_params.WebhookCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Webhook,
        )

    async def update(
        self,
        webhook_id: str,
        *,
        status: Literal["active", "paused", "archived"],
        reason: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Webhook:
        """
        Update a webhook endpoint status.

        **Status transitions:**

        - `active`: Re-enables the endpoint. Use this to:
          - Unpause a paused endpoint
          - Re-enable an endpoint that is backing-off or disabled due to delivery
            failures
          - When re-enabled, any queued notifications will be sent
        - `paused`: Pauses the endpoint. While paused:
          - New notifications are queued (not lost)
          - No delivery attempts are made
          - Use `active` to resume and process queued notifications
        - `archived`: Permanently archives the endpoint (soft delete)

        **Note:** The `backing-off` and `disabled` statuses are system-managed and
        cannot be set directly. These occur automatically when delivery failures are
        detected. Use `active` to re-enable.

        Args:
          status:
              The desired status for the endpoint:

              - `active`: Enable/re-enable the endpoint
              - `paused`: Pause the endpoint (notifications are queued)
              - `archived`: Archive the endpoint (soft delete)

          reason: Optional reason for the status change (for audit purposes)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._patch(
            f"/webhook/{webhook_id}",
            body=await async_maybe_transform(
                {
                    "status": status,
                    "reason": reason,
                },
                webhook_update_params.WebhookUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Webhook,
        )

    async def list(
        self,
        *,
        cursor: str | Omit = omit,
        filter_legal_entity_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookListResponse:
        """
        Get all webhooks

        Args:
          cursor: A cursor string to fetch the next page of results

          filter_legal_entity_id: Pass in a legal entity ID to filter for webhooks under a specific legal entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/webhook",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "filter_legal_entity_id": filter_legal_entity_id,
                    },
                    webhook_list_params.WebhookListParams,
                ),
            ),
            cast_to=WebhookListResponse,
        )


class WebhookResourceWithRawResponse:
    def __init__(self, webhook: WebhookResource) -> None:
        self._webhook = webhook

        self.create = to_raw_response_wrapper(
            webhook.create,
        )
        self.update = to_raw_response_wrapper(
            webhook.update,
        )
        self.list = to_raw_response_wrapper(
            webhook.list,
        )


class AsyncWebhookResourceWithRawResponse:
    def __init__(self, webhook: AsyncWebhookResource) -> None:
        self._webhook = webhook

        self.create = async_to_raw_response_wrapper(
            webhook.create,
        )
        self.update = async_to_raw_response_wrapper(
            webhook.update,
        )
        self.list = async_to_raw_response_wrapper(
            webhook.list,
        )


class WebhookResourceWithStreamingResponse:
    def __init__(self, webhook: WebhookResource) -> None:
        self._webhook = webhook

        self.create = to_streamed_response_wrapper(
            webhook.create,
        )
        self.update = to_streamed_response_wrapper(
            webhook.update,
        )
        self.list = to_streamed_response_wrapper(
            webhook.list,
        )


class AsyncWebhookResourceWithStreamingResponse:
    def __init__(self, webhook: AsyncWebhookResource) -> None:
        self._webhook = webhook

        self.create = async_to_streamed_response_wrapper(
            webhook.create,
        )
        self.update = async_to_streamed_response_wrapper(
            webhook.update,
        )
        self.list = async_to_streamed_response_wrapper(
            webhook.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "Transaction",
    "ACHInfo",
    "CryptoInfo",
    "FeeInfo",
    "FeeInfoRelatedTransaction",
    "MerchantData",
    "MerchantDataLocation",
    "OriginalCurrency",
    "RtpInfo",
    "WireInfo",
]


class ACHInfo(BaseModel):
    """
    Information about the associated ACH transfer if the transaction is an ACH transfer.
    """

    company_discretionary_data: str = FieldInfo(alias="companyDiscretionaryData")
    """
    Optional data field that can be used by the originating company for internal
    purposes
    """

    company_id: str = FieldInfo(alias="companyId")
    """
    The company identification number assigned by the originating depository
    financial institution
    """

    entry_class_code: str = FieldInfo(alias="entryClassCode")
    """
    A three-character code that identifies the type of ACH entry (e.g., PPD for
    Prearranged Payment and Deposit, CCD for Corporate Credit or Debit)
    """

    payment_related_info: str = FieldInfo(alias="paymentRelatedInfo")
    """
    Additional information related to the payment, such as invoice numbers, payment
    references, or other payment-specific details
    """

    receiver_id: str = FieldInfo(alias="receiverId")
    """The unique identifier for the receiver of the ACH transfer"""

    trace_number: str = FieldInfo(alias="traceNumber")
    """
    A unique number assigned by the originating depository financial institution to
    identify the ACH entry
    """

    company_entry_description: Optional[str] = FieldInfo(alias="companyEntryDescription", default=None)
    """
    An optional description of the purpose of the ACH entry as provided by the
    originating company
    """

    counterparty_bank: Optional[str] = FieldInfo(alias="counterpartyBank", default=None)
    """
    The name of the bank or financial institution of the counterparty in the ACH
    transfer
    """


class CryptoInfo(BaseModel):
    """Information populated if this transaction is a crypto on/off-ramp transaction."""

    sender_address: Optional[str] = FieldInfo(alias="senderAddress", default=None)
    """The sender address of the crypto on/off-ramp transaction."""

    tx_hash: Optional[str] = FieldInfo(alias="txHash", default=None)
    """The transaction hash of the crypto on/off-ramp transaction."""


class FeeInfoRelatedTransaction(BaseModel):
    id: str

    amount: float


class FeeInfo(BaseModel):
    """Information populated if this transaction is a fee assessed by Slash."""

    related_transaction: Optional[FeeInfoRelatedTransaction] = FieldInfo(alias="relatedTransaction", default=None)


class MerchantDataLocation(BaseModel):
    """Location details for the merchant/transaction."""

    city: str
    """The city of the merchant."""

    country: str
    """The country of the merchant."""

    state: str
    """The state/province of the merchant."""

    zip: str
    """The ZIP/postal code of the merchant."""


class MerchantData(BaseModel):
    """
    For card transactions, contains description of the transaction as reported by the merchant, merchant category code, and location of the merchant or origin of the transaction.
    For other transactions, this field is undefined.
    """

    category_code: str = FieldInfo(alias="categoryCode")
    """The merchant's category code (MCC)"""

    description: str
    """The raw description provided by the merchant for the transaction."""

    location: Optional[MerchantDataLocation] = None
    """Location details for the merchant/transaction."""


class OriginalCurrency(BaseModel):
    """The original currency of the transaction.

    This is only applicable to transactions. If this field is not sent, the original currency is in USD.
    """

    amount_cents: float = FieldInfo(alias="amountCents")
    """The amount of the transaction in its original currency in cents."""

    code: str
    """The original currency code of the transaction"""

    conversion_rate: float = FieldInfo(alias="conversionRate")
    """
    The conversion rate used to convert the transaction amount from its original
    currency to the account currency. The currency rate is computed at the time of
    the transaction.
    """


class RtpInfo(BaseModel):
    """
    Information about the associated Real-Time Payment (RTP) transfer if the transaction is a real-time transfer.
    """

    counterparty_bank: Optional[str] = FieldInfo(alias="counterpartyBank", default=None)
    """
    The name of the bank or financial institution of the counterparty in the
    real-time transfer
    """

    description: Optional[str] = None
    """Additional description or purpose of the real-time payment"""

    end_to_end_id: Optional[str] = FieldInfo(alias="endToEndId", default=None)
    """
    A unique end-to-end identifier for the real-time payment transaction that
    remains with the payment throughout its lifecycle
    """

    originator_name: Optional[str] = FieldInfo(alias="originatorName", default=None)
    """The name of the originator of the real-time payment"""

    routing_number: Optional[str] = FieldInfo(alias="routingNumber", default=None)
    """The routing number of the counterparty's bank"""


class WireInfo(BaseModel):
    """
    Information about the associated wire transfer if the transaction is a wire transfer.
    """

    business_function_code: str = FieldInfo(alias="businessFunctionCode")
    """
    A code that identifies the business function or purpose of the wire transfer
    (e.g., customer transfer, bank transfer, etc.)
    """

    imad: str
    """
    Incoming Message Authentication Data - a unique identifier assigned by the
    receiving bank for the incoming wire transfer. If the other bank is an account
    with Column N.A., this field is empty.
    """

    omad: str
    """
    Outgoing Message Authentication Data - a unique identifier assigned by the
    originating bank for the outgoing wire transfer. If the other bank is an account
    with Column N.A., this field is empty.
    """

    sender_reference: str = FieldInfo(alias="senderReference")
    """
    A reference number or identifier provided by the sender of the wire transfer for
    tracking purposes
    """

    subtype_code: str = FieldInfo(alias="subtypeCode")
    """A code that identifies the specific type or subtype of the wire transfer"""

    type_code: str = FieldInfo(alias="typeCode")
    """
    A code that identifies the type of wire transfer (e.g., domestic, international)
    """

    counterparty_bank: Optional[str] = FieldInfo(alias="counterpartyBank", default=None)
    """
    The name of the bank or financial institution of the counterparty in the wire
    transfer
    """


class Transaction(BaseModel):
    id: str

    account_id: object = FieldInfo(alias="accountId")
    """The account ID associated with the transaction"""

    account_subtype: Literal["cash", "credit"] = FieldInfo(alias="accountSubtype")
    """
    For charge cards, specifies if the transaction belongs to the cash or credit
    account. For debit accounts, this field is always cash.
    """

    amount_cents: float = FieldInfo(alias="amountCents")
    """The amount of the transaction in cents in USD.

    If the transaction amount is negative, the transaction is a debit. If the
    transaction amount is positive, the transaction is a credit.
    """

    date: str
    """The date in UTC time when the transaction was posted.

    If the transaction is pending or failed, this is the date the transaction was
    created.
    """

    description: str

    detailed_status: Literal[
        "pending",
        "pending_approval",
        "in_review",
        "canceled",
        "failed",
        "settled",
        "declined",
        "refund",
        "reversed",
        "returned",
        "dispute",
    ] = FieldInfo(alias="detailedStatus")
    """- `pending_approval`: Used for any transaction type that is pending approval.

    This does not affect the account's available or posted balances.

    - `pending`: Used for any transaction type that has not posted to the account
      yet, but is affecting the account's available balance.
    - `canceled`: Used for any transaction that was canceled.
    - `failed`: Used for any transaction that failed to post to the account.
    - `settled`: Used for any transaction that has posted to the account, except for
      card refunds, which use the `refund` status, and ACH returns, which use the
      `returned` status.
    - `declined`: Used for card transactions that were declined.
    - `refund`: Used for card transactions that were refunded, increasing the
      account's available and posted balances.
    - `reversed`: Used for card transactions that were authorized, but reversed by
      the merchant.
    - `returned`: Used for ACH returns. In the case of an ACH return, there are two
      possibilities: (1) if the ACH settled and then returned, then there will be
      one transaction with status `settled` for the ACH settling, and one
      transaction with status `returned` for the ACH return. Both of these are
      posted transactions. (2) if the ACH returned before settling, then there will
      only be one transaction with status `canceled`.
    - `dispute`: Used for disputed card transactions.
    """

    status: Literal["pending", "posted", "failed"]
    """
    - `pending`: The transaction is pending and has not been posted to the account
      yet (the account's posted balance doesn't reflect the transaction yet, but the
      account's available balance does).
    - `posted`: The transaction has been posted to the account and the account's
      posted balance and available balances both reflect the transaction.
    - `failed`: The transaction failed to post to the account and no longer affects
      the account's available or posted balances.
    """

    ach_info: Optional[ACHInfo] = FieldInfo(alias="achInfo", default=None)
    """
    Information about the associated ACH transfer if the transaction is an ACH
    transfer.
    """

    approval_reason: Optional[str] = FieldInfo(alias="approvalReason", default=None)
    """The reason for the approval.

    Only exists for card transactions with `detailedStatus` = `pending` or
    `settled`.
    """

    authorized_at: Optional[str] = FieldInfo(alias="authorizedAt", default=None)
    """The UTC timestamp of when the transaction was authorized.

    Only exists for card transactions.
    """

    card_id: Optional[str] = FieldInfo(alias="cardId", default=None)
    """The card ID associated with the transaction.

    If the transaction is not associated with a card, this field is not sent.
    """

    crypto_info: Optional[CryptoInfo] = FieldInfo(alias="cryptoInfo", default=None)
    """Information populated if this transaction is a crypto on/off-ramp transaction."""

    decline_reason: Optional[str] = FieldInfo(alias="declineReason", default=None)
    """The reason for the decline.

    Only exists for card transactions with `detailedStatus` = `declined`.
    """

    fee_info: Optional[FeeInfo] = FieldInfo(alias="feeInfo", default=None)
    """Information populated if this transaction is a fee assessed by Slash."""

    memo: Optional[str] = None
    """The memo associated with the transaction.

    For virtual account transfers, this is the memo provided when creating the
    transfer. For ACH transactions, this is the company entry description. For wire
    transactions, this is the sender reference.
    """

    merchant_data: Optional[MerchantData] = FieldInfo(alias="merchantData", default=None)
    """
    For card transactions, contains description of the transaction as reported by
    the merchant, merchant category code, and location of the merchant or origin of
    the transaction. For other transactions, this field is undefined.
    """

    merchant_description: Optional[str] = FieldInfo(alias="merchantDescription", default=None)
    """DEPRECATED.

    Use `merchantData.description` instead. For card transactions, the description
    of the transaction as reported by the merchant. For other transactions, this
    field is undefined.
    """

    order_id: Optional[str] = FieldInfo(alias="orderId", default=None)
    """
    The order ID, as reported by the merchant, associated with the specific
    transaction.
    """

    original_currency: Optional[OriginalCurrency] = FieldInfo(alias="originalCurrency", default=None)
    """The original currency of the transaction.

    This is only applicable to transactions. If this field is not sent, the original
    currency is in USD.
    """

    provider_authorization_id: Optional[str] = FieldInfo(alias="providerAuthorizationId", default=None)
    """The provider authorization ID for the transaction.

    Only exists for card transactions.
    """

    reference_number: Optional[str] = FieldInfo(alias="referenceNumber", default=None)
    """The reference number provided by Visa for this transaction."""

    rtp_info: Optional[RtpInfo] = FieldInfo(alias="rtpInfo", default=None)
    """
    Information about the associated Real-Time Payment (RTP) transfer if the
    transaction is a real-time transfer.
    """

    virtual_account_id: Optional[str] = FieldInfo(alias="virtualAccountId", default=None)
    """The virtual account ID where the transaction occurred"""

    wire_info: Optional[WireInfo] = FieldInfo(alias="wireInfo", default=None)
    """
    Information about the associated wire transfer if the transaction is a wire
    transfer.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Webhook"]


class Webhook(BaseModel):
    id: str

    created_at: str = FieldInfo(alias="createdAt")

    name: str
    """The name of the webhook"""

    status: Literal["active", "paused", "backing-off", "disabled"]
    """The current status of the webhook endpoint:

    - `active`: The webhook is enabled and receiving events normally.
    - `paused`: The webhook has been paused by the user. Events are queued and will
      be delivered when the endpoint is set back to `active`.
    - `backing-off`: The system is temporarily backing off due to delivery failures.
      Delivery will be automatically retried at `backingOffUntil`. You can also
      immediately re-enable by setting status to `active` via the PATCH endpoint.
    - `disabled`: The webhook has been automatically disabled due to repeated
      delivery failures. Re-enable by setting status to `active` via the PATCH
      endpoint.
    """

    url: str
    """The URL that will receive the webhook payload"""

    archived_at: Optional[str] = FieldInfo(alias="archivedAt", default=None)

    backing_off_until: Optional[str] = FieldInfo(alias="backingOffUntil", default=None)
    """When the system will automatically retry delivery (ISO 8601 timestamp).

    Only present when status is `backing-off`.
    """

    disabled_at: Optional[str] = FieldInfo(alias="disabledAt", default=None)
    """
    When the endpoint was automatically disabled due to repeated failures (ISO 8601
    timestamp). Only present when status is `disabled`.
    """

    paused_at: Optional[str] = FieldInfo(alias="pausedAt", default=None)
    """When the endpoint was paused by the user (ISO 8601 timestamp)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.fdx.customer_retrieve_current_response import CustomerRetrieveCurrentResponse

__all__ = ["CustomersResource", "AsyncCustomersResource"]


class CustomersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CustomersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return CustomersResourceWithStreamingResponse(self)

    def retrieve_current(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerRetrieveCurrentResponse:
        """Get current authenticated customer ID"""
        return self._get(
            "/fdx/customers/current",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerRetrieveCurrentResponse,
        )


class AsyncCustomersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncCustomersResourceWithStreamingResponse(self)

    async def retrieve_current(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerRetrieveCurrentResponse:
        """Get current authenticated customer ID"""
        return await self._get(
            "/fdx/customers/current",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerRetrieveCurrentResponse,
        )


class CustomersResourceWithRawResponse:
    def __init__(self, customers: CustomersResource) -> None:
        self._customers = customers

        self.retrieve_current = to_raw_response_wrapper(
            customers.retrieve_current,
        )


class AsyncCustomersResourceWithRawResponse:
    def __init__(self, customers: AsyncCustomersResource) -> None:
        self._customers = customers

        self.retrieve_current = async_to_raw_response_wrapper(
            customers.retrieve_current,
        )


class CustomersResourceWithStreamingResponse:
    def __init__(self, customers: CustomersResource) -> None:
        self._customers = customers

        self.retrieve_current = to_streamed_response_wrapper(
            customers.retrieve_current,
        )


class AsyncCustomersResourceWithStreamingResponse:
    def __init__(self, customers: AsyncCustomersResource) -> None:
        self._customers = customers

        self.retrieve_current = async_to_streamed_response_wrapper(
            customers.retrieve_current,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .card.card import Card
from .transaction import Transaction

__all__ = ["TransactionRetrieveFeeDetailsResponse", "Item"]


class Item(BaseModel):
    id: str

    account_id: str = FieldInfo(alias="accountId")

    date_charged: str = FieldInfo(alias="dateCharged")

    fee_amount_cents: float = FieldInfo(alias="feeAmountCents")

    fee_type: str = FieldInfo(alias="feeType")

    card: Optional[Card] = None

    original_transaction: Optional[Transaction] = FieldInfo(alias="originalTransaction", default=None)


class TransactionRetrieveFeeDetailsResponse(BaseModel):
    items: List[Item]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .statement_list_params import StatementListParams as StatementListParams
from .statement_list_response import StatementListResponse as StatementListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["AccountListParams"]


class AccountListParams(TypedDict, total=False):
    limit: str
    """The number of items to return (default 100, max 1000)"""

    offset: str
    """The number of items to skip before starting to collect the result set"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .developer_application_data import DeveloperApplicationData
from .developer_application_type import DeveloperApplicationType

__all__ = ["DeveloperApplicationModel"]


class DeveloperApplicationModel(BaseModel):
    id: str

    data: DeveloperApplicationData

    developer_account_id: str = FieldInfo(alias="developerAccountId")

    name: str

    status: Literal["test", "under_review", "production"]

    timestamp: str

    type: DeveloperApplicationType

    oauth_client_id: Optional[str] = FieldInfo(alias="oauthClientId", default=None)

    oauth_client_secret: Optional[str] = FieldInfo(alias="oauthClientSecret", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["WebhookCreateParams"]


class WebhookCreateParams(TypedDict, total=False):
    name: Required[str]

    url: Required[str]
    """The URL that will receive the webhook payload"""

    legal_entity_id: Annotated[str, PropertyInfo(alias="legalEntityId")]
    """The ID of the LegalEntity to create the webhook for.

    You can get this by calling `GET /legal-entity`. This field is required unless
    you are authenticating via API key.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PaginationResponse"]


class PaginationResponse(BaseModel):
    """Response sent when requesting a list of data"""

    count: Optional[float] = None
    """The number of items in the current page of data."""

    next_cursor: Optional[str] = FieldInfo(alias="nextCursor", default=None)
    """The cursor to use to retrieve the next page of data.

    If this is not sent, there is no more data to retrieve.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types import (
    TransferCreateVirtualAccountTransferResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTransfer:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_virtual_account_transfer(self, client: SlashSDK) -> None:
        transfer = client.transfer.create_virtual_account_transfer(
            amount_cents=0,
            destination="destination",
            source="source",
            x_idempotency_key="X-Idempotency-Key",
        )
        assert_matches_type(TransferCreateVirtualAccountTransferResponse, transfer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_virtual_account_transfer_with_all_params(self, client: SlashSDK) -> None:
        transfer = client.transfer.create_virtual_account_transfer(
            amount_cents=0,
            destination="destination",
            source="source",
            x_idempotency_key="X-Idempotency-Key",
            memo="memo",
        )
        assert_matches_type(TransferCreateVirtualAccountTransferResponse, transfer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_virtual_account_transfer(self, client: SlashSDK) -> None:
        response = client.transfer.with_raw_response.create_virtual_account_transfer(
            amount_cents=0,
            destination="destination",
            source="source",
            x_idempotency_key="X-Idempotency-Key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transfer = response.parse()
        assert_matches_type(TransferCreateVirtualAccountTransferResponse, transfer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_virtual_account_transfer(self, client: SlashSDK) -> None:
        with client.transfer.with_streaming_response.create_virtual_account_transfer(
            amount_cents=0,
            destination="destination",
            source="source",
            x_idempotency_key="X-Idempotency-Key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transfer = response.parse()
            assert_matches_type(TransferCreateVirtualAccountTransferResponse, transfer, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTransfer:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_virtual_account_transfer(self, async_client: AsyncSlashSDK) -> None:
        transfer = await async_client.transfer.create_virtual_account_transfer(
            amount_cents=0,
            destination="destination",
            source="source",
            x_idempotency_key="X-Idempotency-Key",
        )
        assert_matches_type(TransferCreateVirtualAccountTransferResponse, transfer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_virtual_account_transfer_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        transfer = await async_client.transfer.create_virtual_account_transfer(
            amount_cents=0,
            destination="destination",
            source="source",
            x_idempotency_key="X-Idempotency-Key",
            memo="memo",
        )
        assert_matches_type(TransferCreateVirtualAccountTransferResponse, transfer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_virtual_account_transfer(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.transfer.with_raw_response.create_virtual_account_transfer(
            amount_cents=0,
            destination="destination",
            source="source",
            x_idempotency_key="X-Idempotency-Key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transfer = await response.parse()
        assert_matches_type(TransferCreateVirtualAccountTransferResponse, transfer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_virtual_account_transfer(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.transfer.with_streaming_response.create_virtual_account_transfer(
            amount_cents=0,
            destination="destination",
            source="source",
            x_idempotency_key="X-Idempotency-Key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transfer = await response.parse()
            assert_matches_type(TransferCreateVirtualAccountTransferResponse, transfer, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.card import spending_constraint_update_full_params, spending_constraint_update_partial_params
from ..._base_client import make_request_options
from ...types.card.spending_constraint import SpendingConstraint

__all__ = ["SpendingConstraintResource", "AsyncSpendingConstraintResource"]


class SpendingConstraintResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpendingConstraintResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return SpendingConstraintResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpendingConstraintResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return SpendingConstraintResourceWithStreamingResponse(self)

    def update_full(
        self,
        card_id: str,
        *,
        country_rule: Optional[spending_constraint_update_full_params.CountryRule] | Omit = omit,
        merchant_category_code_rule: Optional[spending_constraint_update_full_params.MerchantCategoryCodeRule]
        | Omit = omit,
        merchant_category_rule: Optional[spending_constraint_update_full_params.MerchantCategoryRule] | Omit = omit,
        merchant_rule: Optional[spending_constraint_update_full_params.MerchantRule] | Omit = omit,
        spending_rule: Optional[spending_constraint_update_full_params.SpendingRule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpendingConstraint:
        """
        Fully replace a card's spending constraint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return self._put(
            f"/card/{card_id}/spending-constraint",
            body=maybe_transform(
                {
                    "country_rule": country_rule,
                    "merchant_category_code_rule": merchant_category_code_rule,
                    "merchant_category_rule": merchant_category_rule,
                    "merchant_rule": merchant_rule,
                    "spending_rule": spending_rule,
                },
                spending_constraint_update_full_params.SpendingConstraintUpdateFullParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpendingConstraint,
        )

    def update_partial(
        self,
        card_id: str,
        *,
        country_rule: Optional[spending_constraint_update_partial_params.CountryRule] | Omit = omit,
        merchant_category_code_rule: Optional[spending_constraint_update_partial_params.MerchantCategoryCodeRule]
        | Omit = omit,
        merchant_category_rule: Optional[spending_constraint_update_partial_params.MerchantCategoryRule] | Omit = omit,
        merchant_rule: Optional[spending_constraint_update_partial_params.MerchantRule] | Omit = omit,
        spending_rule: Optional[spending_constraint_update_partial_params.SpendingRule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpendingConstraint:
        """
        Apply a partial update to a card's spending constraint while preserving current
        properties applied. Properties set to "null" will be removed from the spending
        constraint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return self._patch(
            f"/card/{card_id}/spending-constraint",
            body=maybe_transform(
                {
                    "country_rule": country_rule,
                    "merchant_category_code_rule": merchant_category_code_rule,
                    "merchant_category_rule": merchant_category_rule,
                    "merchant_rule": merchant_rule,
                    "spending_rule": spending_rule,
                },
                spending_constraint_update_partial_params.SpendingConstraintUpdatePartialParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpendingConstraint,
        )


class AsyncSpendingConstraintResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpendingConstraintResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpendingConstraintResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpendingConstraintResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncSpendingConstraintResourceWithStreamingResponse(self)

    async def update_full(
        self,
        card_id: str,
        *,
        country_rule: Optional[spending_constraint_update_full_params.CountryRule] | Omit = omit,
        merchant_category_code_rule: Optional[spending_constraint_update_full_params.MerchantCategoryCodeRule]
        | Omit = omit,
        merchant_category_rule: Optional[spending_constraint_update_full_params.MerchantCategoryRule] | Omit = omit,
        merchant_rule: Optional[spending_constraint_update_full_params.MerchantRule] | Omit = omit,
        spending_rule: Optional[spending_constraint_update_full_params.SpendingRule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpendingConstraint:
        """
        Fully replace a card's spending constraint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return await self._put(
            f"/card/{card_id}/spending-constraint",
            body=await async_maybe_transform(
                {
                    "country_rule": country_rule,
                    "merchant_category_code_rule": merchant_category_code_rule,
                    "merchant_category_rule": merchant_category_rule,
                    "merchant_rule": merchant_rule,
                    "spending_rule": spending_rule,
                },
                spending_constraint_update_full_params.SpendingConstraintUpdateFullParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpendingConstraint,
        )

    async def update_partial(
        self,
        card_id: str,
        *,
        country_rule: Optional[spending_constraint_update_partial_params.CountryRule] | Omit = omit,
        merchant_category_code_rule: Optional[spending_constraint_update_partial_params.MerchantCategoryCodeRule]
        | Omit = omit,
        merchant_category_rule: Optional[spending_constraint_update_partial_params.MerchantCategoryRule] | Omit = omit,
        merchant_rule: Optional[spending_constraint_update_partial_params.MerchantRule] | Omit = omit,
        spending_rule: Optional[spending_constraint_update_partial_params.SpendingRule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpendingConstraint:
        """
        Apply a partial update to a card's spending constraint while preserving current
        properties applied. Properties set to "null" will be removed from the spending
        constraint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return await self._patch(
            f"/card/{card_id}/spending-constraint",
            body=await async_maybe_transform(
                {
                    "country_rule": country_rule,
                    "merchant_category_code_rule": merchant_category_code_rule,
                    "merchant_category_rule": merchant_category_rule,
                    "merchant_rule": merchant_rule,
                    "spending_rule": spending_rule,
                },
                spending_constraint_update_partial_params.SpendingConstraintUpdatePartialParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpendingConstraint,
        )


class SpendingConstraintResourceWithRawResponse:
    def __init__(self, spending_constraint: SpendingConstraintResource) -> None:
        self._spending_constraint = spending_constraint

        self.update_full = to_raw_response_wrapper(
            spending_constraint.update_full,
        )
        self.update_partial = to_raw_response_wrapper(
            spending_constraint.update_partial,
        )


class AsyncSpendingConstraintResourceWithRawResponse:
    def __init__(self, spending_constraint: AsyncSpendingConstraintResource) -> None:
        self._spending_constraint = spending_constraint

        self.update_full = async_to_raw_response_wrapper(
            spending_constraint.update_full,
        )
        self.update_partial = async_to_raw_response_wrapper(
            spending_constraint.update_partial,
        )


class SpendingConstraintResourceWithStreamingResponse:
    def __init__(self, spending_constraint: SpendingConstraintResource) -> None:
        self._spending_constraint = spending_constraint

        self.update_full = to_streamed_response_wrapper(
            spending_constraint.update_full,
        )
        self.update_partial = to_streamed_response_wrapper(
            spending_constraint.update_partial,
        )


class AsyncSpendingConstraintResourceWithStreamingResponse:
    def __init__(self, spending_constraint: AsyncSpendingConstraintResource) -> None:
        self._spending_constraint = spending_constraint

        self.update_full = async_to_streamed_response_wrapper(
            spending_constraint.update_full,
        )
        self.update_partial = async_to_streamed_response_wrapper(
            spending_constraint.update_partial,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PaySendParams"]


class PaySendParams(TypedDict, total=False):
    amount_cents: Required[Annotated[float, PropertyInfo(alias="amountCents")]]
    """The amount of money to send in cents."""

    slash_handle: Required[Annotated[str, PropertyInfo(alias="slashHandle")]]
    """The username of the SlashHandle to send money to.

    You can get this by asking your recipient for their SlashHandle.
    """

    legal_entity_id: Annotated[str, PropertyInfo(alias="legalEntityId")]
    """The ID of the LegalEntity to send money from.

    You can get this by calling `GET /legal-entity`. This field or `slashHandleId`
    is required unless you are authenticating via API key.
    """

    source_slash_handle_id: Annotated[str, PropertyInfo(alias="sourceSlashHandleId")]
    """The ID of the SlashHandle to send money from.

    You can get this by calling `GET /slash-handle`. This field or `legalEntityId`
    is required unless you are authenticating via API key.
    """
